package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentMiniGameCallbackMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentCallbackDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameCallback;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameCallbackService;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.pojo.vo.AgentRpcVO;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.URIUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.Optional;

@Slf4j
@Service
public class GameTencentMiniGameCallbackServiceImpl extends ServiceImpl<GameTencentMiniGameCallbackMapper, GameTencentMiniGameCallback>
        implements IGameTencentMiniGameCallbackService {

    @Value("${spring.kafka.tencentCallbackTopic}")
    private String tencentCallbackTopic;

    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;

    @Autowired
    private KafkaProducer<String, String> kafkaProducer;

    @Override
    public boolean callback(GameTencentCallbackDTO dto) {
        AgentRpcVO agent = agentRpc.getByTencentAccountId(dto.getAccountId()).getData();
        if (agent == null) {
            log.error("腾讯检测链接数据找不到渠道：{}", JsonUtil.toString(dto));
        }
        GameTencentMiniGameCallback callback = BeanUtil.copy(dto, GameTencentMiniGameCallback.class);
        if (agent != null) {
            callback.setAgentKey(agent.getAgentKey());
            callback.setGameId(agent.getGameId());
            callback.setWechatAppId(agent.getAppId());
        } else {
            callback.setAgentKey("-");
            callback.setGameId(-1L);
        }
        callback.setDay(dto.getClickTime() == null || dto.getClickTime() < 1000 ? LocalDate.now() : DateUtil.secondToLocalDate(dto.getClickTime()));
        save(callback);
        //信息发送到卡夫卡
        try {
            kafkaProducer.send(new ProducerRecord<>(tencentCallbackTopic, dto.getAccountId().toString(), JsonUtil.toString(callback)));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return true;
    }

    @Override
    public String getCallBackUrl(String wechatOpenid, String clickId) {
        try {
            GameTencentMiniGameCallback tencentMiniGameCallback = super.getOne(
                    new LambdaQueryWrapper<GameTencentMiniGameCallback>()
                            .eq(GameTencentMiniGameCallback::getWechatOpenid, wechatOpenid)
                            .and(qw -> qw.eq(GameTencentMiniGameCallback::getImpressionId, clickId)
                                    .or().eq(GameTencentMiniGameCallback::getRequestId, clickId)
                                    .or().eq(GameTencentMiniGameCallback::getClickId, clickId)
                            )
                            .orderByDesc(GameTencentMiniGameCallback::getClickTime)
                            .last("limit 1"));
            String callBack = Optional.ofNullable(tencentMiniGameCallback)
                    .map(GameTencentMiniGameCallback::getCallback)
                    .orElse(null);
            if (Strings.isBlank(callBack)) {
                log.error("匹配监测链接为空, wechatOpenid : {}, clickId : {}", wechatOpenid, clickId);
                return null;
            }
            //返回匹配回传地址
            return URIUtil.decodeURIComponent(callBack);
        } catch (Exception e) {
            log.error("匹配监测链接异常, wechatOpenid : {}, clickId : {}", wechatOpenid, clickId);
        }
        return null;
    }
}
